@@ -17,7 +17,6 @@
 from oslo_config import cfg
 from pecan import hooks
 from six.moves import http_client
-from webob import exc
 
 from ironic.common import context
 from ironic.common import policy
@@ -69,6 +68,7 @@ class ContextHook(hooks.PecanHook):
         # Do not pass any token with context for noauth mode
         auth_token = (None if cfg.CONF.auth_strategy == 'noauth' else
                       headers.get('X-Auth-Token'))
+        is_public_api = state.request.environ.get('is_public_api', False)
 
         creds = {
             'user': headers.get('X-User') or headers.get('X-User-Id'),
@@ -77,16 +77,17 @@ class ContextHook(hooks.PecanHook):
             'domain_name': headers.get('X-User-Domain-Name'),
             'auth_token': auth_token,
             'roles': headers.get('X-Roles', '').split(','),
+            'is_public_api': is_public_api,
         }
 
-        is_admin = policy.enforce('admin_api', creds, creds)
-        is_public_api = state.request.environ.get('is_public_api', False)
-        show_password = policy.enforce('show_password', creds, creds)
+        # TODO(deva): refactor this so enforce is called directly at relevant
+        #             places in code, not globally and for every request
+        show_password = policy.check('show_password', creds, creds)
+        is_admin = policy.check('is_admin', creds, creds)
 
         state.request.context = context.RequestContext(
-            is_admin=is_admin,
-            is_public_api=is_public_api,
             show_password=show_password,
+            is_admin=is_admin,
             **creds)
 
     def after(self, state):
@@ -106,22 +107,6 @@ class RPCHook(hooks.PecanHook):
         state.request.rpcapi = rpcapi.ConductorAPI()
 
 
-class TrustedCallHook(hooks.PecanHook):
-    """Verify that the user has admin rights.
-
-    Checks whether the API call is performed against a public
-    resource or the user has admin privileges in the appropriate
-    tenant, domain or other administrative unit.
-
-    """
-    def before(self, state):
-        ctx = state.request.context
-        if ctx.is_public_api:
-            return
-        policy.enforce('admin_api', ctx.to_dict(), ctx.to_dict(),
-                       do_raise=True, exc=exc.HTTPForbidden)
-
-
 class NoExceptionTracebackHook(hooks.PecanHook):
     """Workaround rpc.common: deserialize_remote_exception.
 
