@@ -82,6 +82,31 @@ class TestApiUtils(base.TestCase):
         values = utils.get_patch_values(patch, path)
         self.assertEqual(['node-x', 'node-y'], values)
 
+    def test_is_path_removed_success(self):
+        patch = [{'path': '/name', 'op': 'remove'}]
+        path = '/name'
+        value = utils.is_path_removed(patch, path)
+        self.assertTrue(value)
+
+    def test_is_path_removed_subpath_success(self):
+        patch = [{'path': '/local_link_connection/switch_id', 'op': 'remove'}]
+        path = '/local_link_connection'
+        value = utils.is_path_removed(patch, path)
+        self.assertTrue(value)
+
+    def test_is_path_removed_similar_subpath(self):
+        patch = [{'path': '/local_link_connection_info/switch_id',
+                  'op': 'remove'}]
+        path = '/local_link_connection'
+        value = utils.is_path_removed(patch, path)
+        self.assertFalse(value)
+
+    def test_is_path_removed_replace(self):
+        patch = [{'path': '/name', 'op': 'replace', 'value': 'node-x'}]
+        path = '/name'
+        value = utils.is_path_removed(patch, path)
+        self.assertFalse(value)
+
     def test_check_for_invalid_fields(self):
         requested = ['field_1', 'field_3']
         supported = ['field_1', 'field_2', 'field_3']
@@ -105,6 +130,34 @@ class TestApiUtils(base.TestCase):
         self.assertRaises(exception.NotAcceptable,
                           utils.check_allow_specify_fields, ['foo'])
 
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allowed_fields_network_interface(self, mock_request):
+        mock_request.version.minor = 20
+        self.assertIsNone(
+            utils.check_allowed_fields(['network_interface']))
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allowed_fields_network_interface_fail(self, mock_request):
+        mock_request.version.minor = 19
+        self.assertRaises(
+            exception.NotAcceptable,
+            utils.check_allowed_fields,
+            ['network_interface'])
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allowed_fields_resource_class(self, mock_request):
+        mock_request.version.minor = 21
+        self.assertIsNone(
+            utils.check_allowed_fields(['resource_class']))
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allowed_fields_resource_class_fail(self, mock_request):
+        mock_request.version.minor = 20
+        self.assertRaises(
+            exception.NotAcceptable,
+            utils.check_allowed_fields,
+            ['resource_class'])
+
     @mock.patch.object(pecan, 'request', spec_set=['version'])
     def test_check_allow_specify_driver(self, mock_request):
         mock_request.version.minor = 16
@@ -116,6 +169,17 @@ class TestApiUtils(base.TestCase):
         self.assertRaises(exception.NotAcceptable,
                           utils.check_allow_specify_driver, ['fake'])
 
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allow_specify_resource_class(self, mock_request):
+        mock_request.version.minor = 21
+        self.assertIsNone(utils.check_allow_specify_resource_class(['foo']))
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_check_allow_specify_resource_class_fail(self, mock_request):
+        mock_request.version.minor = 20
+        self.assertRaises(exception.NotAcceptable,
+                          utils.check_allow_specify_resource_class, ['foo'])
+
     @mock.patch.object(pecan, 'request', spec_set=['version'])
     def test_check_allow_manage_verbs(self, mock_request):
         mock_request.version.minor = 4
@@ -193,6 +257,34 @@ class TestApiUtils(base.TestCase):
         mock_request.version.minor = 17
         utils.check_allow_management_verbs('adopt')
 
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_allow_port_internal_info(self, mock_request):
+        mock_request.version.minor = 18
+        self.assertTrue(utils.allow_port_internal_info())
+        mock_request.version.minor = 17
+        self.assertFalse(utils.allow_port_internal_info())
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_allow_port_advanced_net_fields(self, mock_request):
+        mock_request.version.minor = 19
+        self.assertTrue(utils.allow_port_advanced_net_fields())
+        mock_request.version.minor = 18
+        self.assertFalse(utils.allow_port_advanced_net_fields())
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_allow_network_interface(self, mock_request):
+        mock_request.version.minor = 20
+        self.assertTrue(utils.allow_network_interface())
+        mock_request.version.minor = 19
+        self.assertFalse(utils.allow_network_interface())
+
+    @mock.patch.object(pecan, 'request', spec_set=['version'])
+    def test_allow_resource_class(self, mock_request):
+        mock_request.version.minor = 21
+        self.assertTrue(utils.allow_resource_class())
+        mock_request.version.minor = 20
+        self.assertFalse(utils.allow_resource_class())
+
 
 class TestNodeIdent(base.TestCase):
 
